#!/usr/bin/env python
# -*- coding: utf-8 -*-

PKG_NAME = 'rospeex_core'

try:
    import roslib; roslib.load_manifest(PKG_NAME)
except:
    pass

import os
import logging
import unittest
import ConfigParser

from nose.tools import eq_, raises, nottest
from rospeex_core.sr.client_nict import SpeechRecognitionClient_NICT
from rospeex_core.exceptions import InvalidRequestException
from rospeex_core.exceptions import RequestTimeoutException
from rospeex_core.exceptions import ParameterException
from rospeex_core.exceptions import UnsupportedLanguageException


# set logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s:%(levelname)s:%(filename)s:%(lineno)d:%(message)s')
ch = logging.StreamHandler()
ch.setFormatter(formatter)
logger.addHandler(ch)


class TestSpeechRecognitionClient_NICT(unittest.TestCase):
    def setUp(self):
        base_dir = os.path.dirname(__file__)
        filename = os.path.join(base_dir, 'config.cfg')
        settings = ConfigParser.ConfigParser()
        settings.read(filename)
        self.flac_file = os.path.join(base_dir, settings.get('SpeechRecognition', 'flac_file'))
        self.broken_wav_file = os.path.join(base_dir, settings.get('SpeechRecognition', 'broken_wav_file'))
        self.wav_file = os.path.join(base_dir, settings.get('SpeechRecognition', 'wav_file'))


    @raises(ParameterException)
    def test_request_invalid_audio_format_flac(self):
        request_data = open(self.flac_file, 'rb').read()
        client = SpeechRecognitionClient_NICT()
        client.request(request_data)


    @raises(ParameterException)
    def test_request_invalid_audio_format_broken_wav(self):
        language = 'ja'
        request_data = open(self.broken_wav_file, 'rb').read()
        client = SpeechRecognitionClient_NICT()
        client.request(data=request_data, language=language)


    @raises(ParameterException)
    def test_request_invalid_language(self):
        language = 'hoge'
        request_data = open(self.wav_file, 'rb').read()
        client = SpeechRecognitionClient_NICT()
        client.request(data=request_data, language=language)


    @raises(RequestTimeoutException)
    def test_request_invalid_request_timeout(self):
        language = 'ja'
        request_data = open(self.wav_file, 'rb').read()
        client = SpeechRecognitionClient_NICT()
        client.request(data=request_data, language=language, timeout=1)


    def test_request_valid_language(self):
        for language in SpeechRecognitionClient_NICT.LANGUAGES:
            request_data = open(self.wav_file, 'rb').read()
            client = SpeechRecognitionClient_NICT()
            msg = client.request(data=request_data, language=language)
            logger.info(msg)


if __name__ == '__main__':
    import rosunit
    test_class = TestSpeechRecognitionClient_NICT
    rosunit.unitrun(PKG_NAME, 'speech_recognition_client_nict', test_class, None, coverage_packages=['rospeex.sr'])
