#!/usr/bin/env python
# -*- coding: utf-8 -*-

PKG_NAME = 'rospeex_core'

try:
    import roslib; roslib.load_manifest(PKG_NAME)
except:
    pass

import os
import logging
import unittest
from nose.tools import eq_, raises
import ConfigParser

from rospeex_core.ss.client_nict import SpeechSynthesisClient_NICT
from rospeex_core.exceptions import ParameterException
from rospeex_core.exceptions import InvalidResponseException
from rospeex_core.exceptions import InvalidRequestException
from rospeex_core.exceptions import RequestTimeoutException
from rospeex_core.exceptions import SpeechSynthesisException
from rospeex_core.exceptions import UnsupportedLanguageException

# set logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s:%(levelname)s:%(filename)s:%(lineno)d:%(message)s')
ch = logging.StreamHandler()
ch.setFormatter(formatter)
logger.addHandler(ch)


class TestSpeechSynthesisClient_NICT(unittest.TestCase):
    def setUp(self):
        base_dir = os.path.dirname(__file__)
        settings = ConfigParser.ConfigParser()
        filename = os.path.join(base_dir, 'config.cfg')
        settings.read(filename)
        self.big_text_file = os.path.join(base_dir, settings.get('SpeechSynthesis', 'big_text_file'))

        self.big_text_data = None
        with open(self.big_text_file, 'r') as f:
            self.big_text_data = f.read()


    @raises(ParameterException)
    def test_request_invalid_message(self):
        message    = None
        language   = 'ja'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    @raises(InvalidResponseException)
    def test_request_invalid_message_charactor(self):
        message    = '<'
        language   = 'ja'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    @raises(UnsupportedLanguageException)
    def test_request_invalid_language(self):
        message    = 'hello'
        language   = 'de'        # deutsch
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    @raises(InvalidRequestException)
    def test_request_invalid_url(self):
        message    = 'hello'
        language   = 'en'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.URL = 'http://rospeex.ucri.jgn-x.jp/nauth_json/jsServices/hoge'
        client.request(message, language, voice_font, timeout)


    @raises(ParameterException)
    def test_request_invalid_voice_font(self):
        message    = 'hello'
        language   = 'ja'
        voice_font = 'hogehoge'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    @raises(RequestTimeoutException)
    def test_request_server_timeout_prev(self):
        message    = self.big_text_data
        language   = 'en'
        voice_font = '*'
        timeout    = 1
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    @raises(RequestTimeoutException)
    def test_request_server_timeout_post(self):
        message    = self.big_text_data
        language   = 'en'
        voice_font = '*'
        timeout    = 2
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    def test_request_valid_big_message(self):
        message    = self.big_text_data        # alice in wonder land
        language   = 'en'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    def test_request_valid_japanese_message(self):
        message    = u'こんにちは'
        language   = 'ja'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


    def test_request_valid_english_message(self):
        message    = 'hello everyone.'
        language   = 'en'
        voice_font = '*'
        timeout    = 100
        client = SpeechSynthesisClient_NICT()
        client.request(message, language, voice_font, timeout)


if __name__ == '__main__':
    import rosunit
    test_class = TestSpeechSynthesisClient_NICT
    rosunit.unitrun(PKG_NAME, 'speech_synthesis_client_nict', test_class, None, coverage_packages=['rospeex.ss'])
